from Products.CMFCore.utils import getToolByName

def enableExplicitMode():
    """ZCML startup hook to put the ruleset registry into explict mode.
    This means we require people to declare ruleset types before using them.
    """
    from z3c.caching.registry import getGlobalRulesetRegistry
    registry = getGlobalRulesetRegistry()
    if registry is not None:
        registry.explicit = True

def importVarious(context):

    if not context.readDataFile('plone.app.caching.txt'):
        return

    site = context.getSite()

    error_log = getToolByName(site, 'error_log')

    properties = error_log.getProperties()
    ignored = properties.get('ignored_exceptions', ())

    modified = False
    for exceptionName in ('Intercepted',):
        if exceptionName not in ignored:
            ignored += (exceptionName,)
            modified = True

    if modified:
        error_log.setProperties(properties.get('keep_entries', 10),
                                properties.get('copy_to_zlog', True),
                                ignored)



import unittest2 as unittest

import transaction;

from plone.testing.z2 import Browser

from plone.app.testing import TEST_USER_ID, TEST_USER_NAME, TEST_USER_PASSWORD
from plone.app.testing import setRoles
from plone.app.textfield.value import RichTextValue

from zope.component import getUtility

from zope.globalrequest import setRequest

from plone.registry.interfaces import IRegistry
from plone.caching.interfaces import ICacheSettings

from plone.app.caching.testing import PLONE_APP_CACHING_FUNCTIONAL_TESTING

class TestOperationParameters(unittest.TestCase):
    """This test aims to test the effect of changing various caching operation
    parameters.
    """

    layer = PLONE_APP_CACHING_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer['app']
        self.portal = self.layer['portal']

        setRequest(self.portal.REQUEST)

        self.registry = getUtility(IRegistry)
        self.cacheSettings = self.registry.forInterface(ICacheSettings)
        self.cacheSettings.enabled = True

    def tearDown(self):
        setRequest(None)

    def test_anon_only(self):

        # Add folder content
        setRoles(self.portal, TEST_USER_ID, ('Manager',))
        self.portal.invokeFactory('Folder', 'f1')
        self.portal['f1'].title = u"Folder one"
        self.portal['f1'].description = u"Folder one description"
        self.portal['f1'].reindexObject()

        # Add page content
        self.portal['f1'].invokeFactory('Document', 'd1')
        self.portal['f1']['d1'].title = u"Document one"
        self.portal['f1']['d1'].description = u"Document one description"
        testText = "Testing... body one"
        self.portal['f1']['d1'].text = RichTextValue(
            testText,
            'text/plain',
            'text/html'
        )
        self.portal['f1']['d1'].reindexObject()

        # Publish the folder and page
        self.portal.portal_workflow.doActionFor(self.portal['f1'], 'publish')
        self.portal.portal_workflow.doActionFor(self.portal['f1']['d1'], 'publish')

        # Set pages to have weak caching and test anonymous

        self.cacheSettings.operationMapping = {'plone.content.itemView': 'plone.app.caching.weakCaching'}
        transaction.commit()

        # View the page as anonymous
        browser = Browser(self.app)
        browser.open(self.portal['f1']['d1'].absolute_url())
        self.assertEqual('plone.content.itemView', browser.headers['X-Cache-Rule'])
        self.assertEqual('plone.app.caching.weakCaching', browser.headers['X-Cache-Operation'])
        self.assertTrue(testText in browser.contents)
        self.assertEqual('max-age=0, must-revalidate, private', browser.headers['Cache-Control'])

        # Set pages to have moderate caching so that we can see the difference
        # between logged in and anonymous

        self.cacheSettings.operationMapping = {'plone.content.itemView': 'plone.app.caching.moderateCaching'}
        self.registry['plone.app.caching.moderateCaching.smaxage'] = 60
        self.registry['plone.app.caching.moderateCaching.vary'] = 'X-Anonymous'
        self.registry['plone.app.caching.moderateCaching.anonOnly'] = True

        transaction.commit()

        # View the page as anonymous
        browser = Browser(self.app)
        browser.open(self.portal['f1']['d1'].absolute_url())
        self.assertEqual('plone.content.itemView', browser.headers['X-Cache-Rule'])
        self.assertEqual('plone.app.caching.moderateCaching', browser.headers['X-Cache-Operation'])
        self.assertTrue(testText in browser.contents)
        self.assertEqual('max-age=0, s-maxage=60, must-revalidate', browser.headers['Cache-Control'])
        self.assertEqual('X-Anonymous', browser.headers['Vary'])
        self.assertFalse('Etag' in browser.headers)

        # View the page as logged-in
        browser = Browser(self.app)
        browser.addHeader('Authorization', 'Basic %s:%s' % (TEST_USER_NAME, TEST_USER_PASSWORD,))
        browser.open(self.portal['f1']['d1'].absolute_url())
        self.assertEqual('plone.content.itemView', browser.headers['X-Cache-Rule'])
        self.assertEqual('plone.app.caching.moderateCaching', browser.headers['X-Cache-Operation'])
        self.assertTrue(testText in browser.contents)
        self.assertEqual('max-age=0, must-revalidate, private', browser.headers['Cache-Control'])
        self.assertTrue('Etag' in browser.headers)

        # Set pages to have strong caching so that we can see the difference
        # between logged in and anonymous

        self.cacheSettings.operationMapping = {'plone.content.itemView': 'plone.app.caching.strongCaching'}
        self.registry['plone.app.caching.strongCaching.vary'] = 'X-Anonymous'
        self.registry['plone.app.caching.strongCaching.anonOnly'] = True
        transaction.commit()

        # View the page as anonymous
        browser = Browser(self.app)
        browser.open(self.portal['f1']['d1'].absolute_url())
        self.assertEqual('plone.content.itemView', browser.headers['X-Cache-Rule'])
        self.assertEqual('plone.app.caching.strongCaching', browser.headers['X-Cache-Operation'])
        self.assertTrue(testText in browser.contents)
        self.assertEqual('max-age=86400, proxy-revalidate, public', browser.headers['Cache-Control'])
        self.assertEqual('X-Anonymous', browser.headers['Vary'])
        self.assertFalse('Etag' in browser.headers)

        # View the page as logged-in
        browser = Browser(self.app)
        browser.addHeader('Authorization', 'Basic %s:%s' % (TEST_USER_NAME, TEST_USER_PASSWORD,))
        browser.open(self.portal['f1']['d1'].absolute_url())
        self.assertEqual('plone.content.itemView', browser.headers['X-Cache-Rule'])
        self.assertEqual('plone.app.caching.strongCaching', browser.headers['X-Cache-Operation'])
        self.assertTrue(testText in browser.contents)
        self.assertEqual('max-age=0, must-revalidate, private', browser.headers['Cache-Control'])
        self.assertTrue('Etag' in browser.headers)

        # Check an edge case that has had a problem in the past:
        # setting strongCaching maxage to zero.

        self.registry['plone.app.caching.strongCaching.maxage'] = 0
        self.registry['plone.app.caching.strongCaching.smaxage'] = 60
        transaction.commit()

        # View the page as anonymous
        browser = Browser(self.app)
        browser.open(self.portal['f1']['d1'].absolute_url())
        self.assertEqual('max-age=0, s-maxage=60, must-revalidate', browser.headers['Cache-Control'])



import unittest2 as unittest
from plone.testing.zca import UNIT_TESTING

import os
import time
import datetime
import DateTime

from dateutil.tz import tzlocal

from persistent.TimeStamp import TimeStamp

from zope.component import provideAdapter
from z3c.caching.interfaces import ILastModified

from plone.app.caching import lastmodified

class FauxDataManager(object):

    def setstate(self, object):
        pass

    def oldstate(self, obj, tid):
        pass

    def register(self, object):
        pass

class TestLastModified(unittest.TestCase):

    layer = UNIT_TESTING

    def setUp(self):
        provideAdapter(lastmodified.PageTemplateDelegateLastModified)
        provideAdapter(lastmodified.FSPageTemplateDelegateLastModified)
        provideAdapter(lastmodified.OFSFileLastModified)
        provideAdapter(lastmodified.FSObjectLastModified)
        provideAdapter(lastmodified.CatalogableDublinCoreLastModified)
        provideAdapter(lastmodified.DCTimesLastModified)
        provideAdapter(lastmodified.ResourceLastModified)

    def test_PageTemplateDelegateLastModified(self):
        from persistent import Persistent
        from Acquisition import Explicit

        class Dummy(Persistent, Explicit):
            _p_mtime = None

        provideAdapter(lastmodified.PersistentLastModified, adapts=(Dummy,))

        d = Dummy()

        from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
        zpt = ZopePageTemplate('zpt').__of__(d)
        self.assertEqual(None, ILastModified(zpt)())


        timestamp = 987654321.0 # time stamp (in UTC)
        # equivalent in local time, which is what the last-modified adapter
        # should return
        mod = datetime.datetime.fromtimestamp(timestamp, tzlocal())

        d._p_mtime = 987654321.0
        self.assertEqual(mod, ILastModified(zpt)())

    def test_FSPageTemplateDelegateLastModified(self):
        from persistent import Persistent
        from Acquisition import Explicit

        class Dummy(Persistent, Explicit):
            _p_mtime = None

        provideAdapter(lastmodified.PersistentLastModified, adapts=(Dummy,))

        d = Dummy()

        from Products.CMFCore.FSPageTemplate import FSPageTemplate
        zpt = FSPageTemplate('zpt', __file__).__of__(d)
        self.assertEqual(None, ILastModified(zpt)())


        timestamp = 987654321.0 # time stamp (in UTC)
        # equivalent in local time, which is what the last-modified adapter
        # should return
        mod = datetime.datetime.fromtimestamp(timestamp, tzlocal())

        d._p_mtime = 987654321.0
        self.assertEqual(mod, ILastModified(zpt)())

    def test_OFSFileLastModified_File(self):
        from OFS.Image import File

        dummy = File('dummy', 'Dummy', 'data')
        self.assertEqual(None, ILastModified(dummy)())

        timestamp = 987654321.0 # time stamp (in UTC)
        ts = TimeStamp(*time.gmtime(timestamp)[:6]) # corresponding TimeStamp

        # equivalent in local time, which is what the last-modified adapter
        # should return
        mod = datetime.datetime.fromtimestamp(timestamp, tzlocal())

        dummy._p_jar = FauxDataManager()
        dummy._p_serial = repr(ts)
        self.assertEqual(mod, ILastModified(dummy)())

    def test_OFSFileLastModified_Image(self):
        from OFS.Image import Image

        dummy = Image('dummy', 'Dummy', 'data')
        self.assertEqual(None, ILastModified(dummy)())

        timestamp = 987654321.0 # time stamp (in UTC)
        ts = TimeStamp(*time.gmtime(timestamp)[:6]) # corresponding TimeStamp

        # equivalent in local time, which is what the last-modified adapter
        # should return
        mod = datetime.datetime.fromtimestamp(timestamp, tzlocal())

        dummy._p_jar = FauxDataManager()
        dummy._p_serial = repr(ts)
        self.assertEqual(mod, ILastModified(dummy)())

    def test_FSObjectLastModified_FSFile(self):
        from Products.CMFCore.FSFile import FSFile

        dummy = FSFile('dummy', __file__)

        modtime = float(os.path.getmtime(__file__))
        mod = datetime.datetime.fromtimestamp(modtime, tzlocal())

        format = "%y%m%d%H%M%s" # see note in test_FSObjectLastModified_FSImage
        self.assertEqual(mod.strftime(format), ILastModified(dummy)().strftime(format))

    def test_FSObjectLastModified_FSImage(self):
        from Products.CMFCore.FSImage import FSImage

        dummy = FSImage('dummy', __file__) # not really an image, but anyway
        modtime = float(os.path.getmtime(__file__))
        mod = datetime.datetime.fromtimestamp(modtime, tzlocal())
        # different filesystems seem to handle datetime differently. some use microseconds
        # and others don't so to make jenkins happy lets omit the microseconds factor
        format = "%y%m%d%H%M%s"
        self.assertEqual(mod.strftime(format), ILastModified(dummy)().strftime(format))

    def test_CatalogableDublinCoreLastModified(self):
        from Products.CMFCore.interfaces import ICatalogableDublinCore
        from zope.interface import implements

        class Dummy(object):
            implements(ICatalogableDublinCore)

            _mod = None

            def modified(self):
                if self._mod is not None: return DateTime.DateTime(self._mod)
                return None

        d = Dummy()

        self.assertEqual(None, ILastModified(d)())

        d._mod = datetime.datetime(2001, 4, 19, 12, 25, 21, 120000)
        self.assertEqual(d._mod, ILastModified(d)())

    def test_DCTimesLastModified(self):
        try:
            from zope.dublincore.interfaces import IDCTimes
        except ImportError:
            return
        from zope.interface import implements

        class Dummy(object):
            implements(IDCTimes)

            _mod = None

            @property
            def modified(self):
                return self._mod

        d = Dummy()

        self.assertEqual(None, ILastModified(d)())

        d._mod = datetime.datetime(2001, 4, 19, 12, 25, 21, 120000)
        self.assertEqual(d._mod, ILastModified(d)())

    def test_ResourceLastModified_zope_app(self):
        from zope.browserresource.file import FileResource
        from zope.browserresource.file import File

        class DummyRequest(dict):
            pass

        request = DummyRequest()

        f = File(__file__, 'test_lastmodified.py')
        r = FileResource(f, request)

        modtime = float(os.path.getmtime(__file__))
        mod = datetime.datetime.fromtimestamp(modtime, tz=tzlocal())

        self.assertEqual(mod, ILastModified(r)())

    def test_ResourceLastModified_Five(self):
        from Products.Five.browser.resource import FileResource
        from zope.browserresource.file import File

        class DummyRequest(dict):
            pass

        request = DummyRequest()

        f = File(__file__, 'test_lastmodified.py')  # not really an image
        r = FileResource(f, request)

        modtime = float(os.path.getmtime(__file__))
        mod = datetime.datetime.fromtimestamp(modtime, tz=tzlocal())

        self.assertEqual(mod, ILastModified(r)())



import re
import datetime

from zope.interface import implements
from zope.component import getUtility
from zope.component import getUtilitiesFor
from zope.component import queryUtility

from zope.publisher.interfaces import IPublishTraverse
from zope.publisher.interfaces import NotFound

from zope.ramcache.interfaces.ram import IRAMCache

from plone.memoize.instance import memoize

from plone.registry.interfaces import IRegistry

from z3c.caching.interfaces import IRulesetType
from z3c.caching.registry import enumerateTypes

from plone.protect import CheckAuthenticator

from plone.caching.interfaces import ICacheSettings
from plone.caching.interfaces import ICachingOperationType

from plone.cachepurging.interfaces import IPurger
from plone.cachepurging.interfaces import ICachePurgingSettings

from plone.cachepurging.utils import isCachePurgingEnabled
from plone.cachepurging.utils import getPathsToPurge
from plone.cachepurging.utils import getURLsToPurge

from plone.app.caching.interfaces import IPloneCacheSettings
from plone.app.caching.interfaces import ICacheProfiles
from plone.app.caching.interfaces import _
from plone.app.caching.browser.edit import EditForm

from Products.GenericSetup.interfaces import BASE, EXTENSION
from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage

# Borrowed from zope.schema to avoid an import of a private name
_isuri = re.compile(
    r"[a-zA-z0-9+.-]+:"   # scheme
    r"\S*$"               # non space (should be pickier)
    ).match


class BaseView(object):

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        self.update()
        if self.request.response.getStatus() not in (301, 302):
            return self.render()
        return ''

    def update(self):
        self.errors = {}

        self.registry = getUtility(IRegistry)
        self.settings = self.registry.forInterface(ICacheSettings)
        self.ploneSettings = self.registry.forInterface(IPloneCacheSettings)
        self.purgingSettings = self.registry.forInterface(ICachePurgingSettings)
        self.ramCache = queryUtility(IRAMCache)

        if self.request.method == 'POST':
            CheckAuthenticator(self.request)
            return True
        return False

    def render(self):
        return self.index()

    @property
    @memoize
    def purgingEnabled(self):
        return isCachePurgingEnabled()


class ControlPanel(BaseView):
    """Control panel view
    """

    implements(IPublishTraverse)

    # Used by the publishTraverse() method - see below
    editGlobal = False
    editRuleset = False
    editOperationName = None
    editRulesetName = None

    def publishTraverse(self, request, name):
        """Allow the following types of URLs:

        /
            Render the standard control panel (no publish traverse invoked)

        /edit-operation-global/${operation-name}
            Render an edit form for global operation parameters

        /edit-operation-ruleset/${operation-name}/${ruleset-name}
            Render an edit form for per-ruleset operation parameters
        """

        # Step 1 - find which type of editing we want to do
        if not self.editGlobal and not self.editRuleset:
            if name == 'edit-operation-global':
                self.editGlobal = True
            elif name == 'edit-operation-ruleset':
                self.editRuleset = True
            else:
                raise NotFound(self, name)
            return self  # traverse again to get operation name

        # Step 2 - get operation name
        if (self.editGlobal or self.editRuleset) and not self.editOperationName:
            self.editOperationName = name

            if self.editGlobal:

                operation = queryUtility(ICachingOperationType, name=self.editOperationName)
                if operation is None:
                    raise NotFound(self, operation)

                return EditForm(self.context, self.request, self.editOperationName, operation)
            elif self.editRuleset:
                return self  # traverse again to get ruleset name
            else:
                raise NotFound(self, name)

        # Step 3 - if this is ruleset traversal, get the ruleset name
        if self.editRuleset and self.editOperationName and not self.editRulesetName:
            self.editRulesetName = name

            operation = queryUtility(ICachingOperationType, name=self.editOperationName)
            if operation is None:
                raise NotFound(self, self.operationName)

            rulesetType = queryUtility(IRulesetType, name=self.editRulesetName)
            if rulesetType is None:
                raise NotFound(self, self.editRulesetName)

            return EditForm(self.context, self.request,
                            self.editOperationName, operation,
                            self.editRulesetName, rulesetType)

        raise NotFound(self, name)

    def update(self):
        if super(ControlPanel, self).update():
            if 'form.button.Save' in self.request.form:
                self.processSave()
            elif 'form.button.Cancel' in self.request.form:
                self.request.response.redirect(
                    "%s/plone_control_panel" % self.context.absolute_url())

    def processSave(self):

        form = self.request.form

        # Form data
        enabled = form.get('enabled', False)
        contentTypesMap = form.get('contenttypes', {})
        templatesMap = form.get('templates', {})
        operations = form.get('operations', {})

        purgingEnabled = form.get('purgingEnabled', False)
        cachingProxies = tuple(form.get('cachingProxies', ()))
        purgedContentTypes = tuple(form.get('purgedContentTypes', ()))
        virtualHosting = form.get('virtualHosting', False)
        domains = tuple(form.get('domains', ()))

        ramCacheMaxEntries = form.get('ramCacheMaxEntries', None)
        ramCacheMaxAge = form.get('ramCacheMaxAge', None)
        ramCacheCleanupInterval = form.get('ramCacheCleanupInterval', None)

        # Settings

        operationMapping = {}
        contentTypeRulesetMapping = {}
        templateRulesetMapping = {}

        # Process mappings and validate

        for ruleset, operation in operations.items():

            if not ruleset or not operation:
                continue

            if isinstance(ruleset, unicode):  # should be ASCII
                ruleset = ruleset.encode('utf-8')

            if isinstance(operation, unicode):  # should be ASCII
                operation = operation.encode('utf-8')

            ruleset = ruleset.replace('-', '.')
            operationMapping[ruleset] = operation

        for ruleset, contentTypes in contentTypesMap.items():

            if not ruleset:
                continue

            if isinstance(ruleset, unicode):  # should be ASCII
                ruleset = ruleset.encode('utf-8')

            ruleset = ruleset.replace('-', '.')

            for contentType in contentTypes:

                if not contentType:
                    continue

                if isinstance(contentType, unicode):  # should be ASCII
                    contentType = contentType.encode('utf-8')

                if contentType in contentTypeRulesetMapping:
                    self.errors.setdefault('contenttypes', {})[ruleset] = \
                        _(u"Content type ${contentType} is already mapped to the rule ${ruleset}.",
                            mapping={
                                'contentType': self.contentTypesLookup.get(contentType, {}).get('title', contentType),  # noqa
                                'ruleset': contentTypeRulesetMapping[contentType]})
                else:
                    contentTypeRulesetMapping[contentType] = ruleset

        for ruleset, templates in templatesMap.items():

            if not ruleset:
                continue

            if isinstance(ruleset, unicode):  # should be ASCII
                ruleset = ruleset.encode('utf-8')

            ruleset = ruleset.replace('-', '.')

            for template in templates:

                template = template.strip()

                if not template:
                    continue

                if isinstance(template, unicode):  # should be ASCII
                    template = template.encode('utf-8')

                if template in templateRulesetMapping:
                    self.errors.setdefault('templates', {})[ruleset] = \
                        _(u"Template ${template} is already mapped to the rule ${ruleset}.",
                            mapping={
                                'template': template,
                                'ruleset': templateRulesetMapping[template]})
                else:
                    templateRulesetMapping[template] = ruleset

        # Validate purging settings

        for cachingProxy in cachingProxies:
            if not _isuri(cachingProxy):
                self.errors['cachingProxies'] = _(u"Invalid URL: ${url}", mapping={'url': cachingProxy})  # noqa

        for domain in domains:
            if not _isuri(domain):
                self.errors['domain'] = _(u"Invalid URL: ${url}", mapping={'url': domain})

        # RAM cache settings

        try:
            ramCacheMaxEntries = int(ramCacheMaxEntries)
        except (ValueError, TypeError,):
            self.errors['ramCacheMaxEntries'] = _(u"An integer is required.")
        else:
            if ramCacheMaxEntries < 0:
                self.errors['ramCacheMaxEntries'] = _(u"A positive number is required.")

        try:
            ramCacheMaxAge = int(ramCacheMaxAge)
        except (ValueError, TypeError,):
            self.errors['ramCacheMaxAge'] = _(u"An integer is required.")
        else:
            if ramCacheMaxAge < 0:
                self.errors['ramCacheMaxAge'] = _(u"A positive number is required.")

        try:
            ramCacheCleanupInterval = int(ramCacheCleanupInterval)
        except (ValueError, TypeError,):
            self.errors['ramCacheCleanupInterval'] = _(u"An integer is required.")
        else:
            if ramCacheMaxAge < 0:
                self.errors['ramCacheCleanupInterval'] = _(u"A positive number is required.")

        # Check for errors
        if self.errors:
            IStatusMessage(self.request).addStatusMessage(_(u"There were errors."), "error")
            return

        # Save settings
        self.settings.enabled = enabled
        self.settings.operationMapping = operationMapping

        self.ploneSettings.templateRulesetMapping = templateRulesetMapping
        self.ploneSettings.contentTypeRulesetMapping = contentTypeRulesetMapping
        self.ploneSettings.purgedContentTypes = purgedContentTypes

        self.purgingSettings.enabled = purgingEnabled
        self.purgingSettings.cachingProxies = cachingProxies
        self.purgingSettings.virtualHosting = virtualHosting
        self.purgingSettings.domains = domains

        self.ramCache.update(ramCacheMaxEntries, ramCacheMaxAge, ramCacheCleanupInterval)

        IStatusMessage(self.request).addStatusMessage(_(u"Changes saved."), "info")

    # Rule types - used as the index column
    @property
    @memoize
    def ruleTypes(self):
        types = []
        for type_ in enumerateTypes():
            types.append(dict(name=type_.name,
                              title=type_.title or type_.name,
                              description=type_.description,
                              safeName=type_.name.replace('.', '-')))
        types.sort(lambda x, y: cmp(x['title'], y['title']))
        return types

    # Safe access to the main mappings, which may be None - we want to treat
    # that as {} to make TAL expressions simpler. We also need the safeName
    # equivalent name for the key

    @property
    def operationMapping(self):
        return dict(
            [(k.replace('.', '-'), v,)
             for k, v in (self.settings.operationMapping or {}).items()]
        )

    @property
    def templateMapping(self):
        return dict(
            [(k, v.replace('.', '-'),)
             for k, v in (self.ploneSettings.templateRulesetMapping or {}).items()]
        )

    @property
    def contentTypeMapping(self):
        return dict(
            [(k, v.replace('.', '-'),)
             for k, v in (self.ploneSettings.contentTypeRulesetMapping or {}).items()]
        )

    # Type lookups (for accessing settings)

    @property
    @memoize
    def operationTypesLookup(self):
        lookup = {}
        for name, type_ in getUtilitiesFor(ICachingOperationType):
            lookup[name] = dict(
                name=name,
                title=type_.title,
                description=getattr(type_, 'description', ''),
                sort=getattr(type_, 'sort', 100),
                prefix=getattr(type_, 'prefix', None),
                options=getattr(type_, 'options', ()),
                hasOptions=self.hasGlobalOptions(type_),
                type=type_,
            )
        return lookup

    @property
    @memoize
    def contentTypesLookup(self):
        types = {}
        portal_types = getToolByName(self.context, 'portal_types')
        for fti in portal_types.objectValues():
            types[fti.id] = dict(title=fti.title or fti.id, description=fti.description)
        return types

    # Sorted lists (e.g. for drop-downs)

    @property
    @memoize
    def operationTypes(self):
        operations = [v for k, v in self.operationTypesLookup.items()]
        operations.sort(lambda x, y: (cmp(x['sort'], y['sort']) or cmp(x['title'], y['title'])))
        return operations

    @property
    @memoize
    def contentTypes(self):
        types = [
            dict(
                name=name,
                title=info['title'],
                description=info['description']
            ) for name, info in self.contentTypesLookup.items()
        ]
        types.sort(lambda x, y: cmp(x['title'], y['title']))
        return types

    # We store template and content type mappings as template -> ruleset and
    # content type -> ruleset. In the UI, we reverse this, so that the user
    # enters a list of templates and selects a set of content types for each
    # ruleset. This is more natural (whereas the storage is more efficient).
    # These mappings support that UI

    @property
    @memoize
    def reverseContentTypeMapping(self):
        mapping = {}
        for contentType, ruleset in self.contentTypeMapping.items():
            mapping.setdefault(ruleset, []).append(contentType)
        return mapping

    @property
    @memoize
    def reverseTemplateMapping(self):
        mapping = {}
        for template, ruleset in self.templateMapping.items():
            mapping.setdefault(ruleset, []).append(template)
        return mapping

    # For the ruleset mappings page, we need to know whether a particular
    # operation has global and per-ruleset parameters. If there is at least
    # one option set, we consider it to have options.

    def hasGlobalOptions(self, operationType):
        prefix = getattr(operationType, 'prefix', None)
        options = getattr(operationType, 'options', ())

        if not options or not prefix:
            return False

        for option in options:
            if '%s.%s' % (prefix, option,) in self.registry:
                return True

        return False

    def hasRulesetOptions(self, operationType, ruleset):
        prefix = operationType.prefix
        options = operationType.options

        if not options or not prefix:
            return False

        for option in options:
            if '%s.%s.%s' % (prefix, ruleset, option,) in self.registry:
                return True

        return False


class Import(BaseView):
    """The import control panel
    """

    def update(self):
        if super(Import, self).update():
            if 'form.button.Import' in self.request.form:
                self.processImport()

    def processImport(self):
        profile = self.request.form.get('profile', None)
        snapshot = self.request.form.get('snapshot', True)

        if not profile:
            self.errors['profile'] = _(u"You must select a profile to import.")

        if self.errors:
            IStatusMessage(self.request).addStatusMessage(_(u"There were errors."), "error")
            return

        portal_setup = getToolByName(self.context, 'portal_setup')

        # Create a snapshot
        if snapshot:
            snapshotId = "plone.app.caching.beforeimport.%s" % (
                datetime.datetime.now().isoformat().replace(':', '.'))
            portal_setup.createSnapshot(snapshotId)

        # Import the new profile
        portal_setup.runAllImportStepsFromProfile("profile-%s" % profile)

        IStatusMessage(self.request).addStatusMessage(_(u"Import complete."), "info")

    @property
    @memoize
    def profiles(self):
        portal_setup = getToolByName(self.context, 'portal_setup')
        return [profile for profile in portal_setup.listProfileInfo(ICacheProfiles)
                if profile.get('type', BASE) == EXTENSION and profile.get('for') is not None]


class Purge(BaseView):
    """The purge control panel
    """
    def update(self):
        self.purgeLog = []

        if super(Purge, self).update():
            if 'form.button.Purge' in self.request.form:
                self.processPurge()

    def processPurge(self):
        urls = self.request.form.get('urls', [])
        sync = self.request.form.get('synchronous', True)

        if not urls:
            self.errors['urls'] = _(u"No URLs or paths entered.")

        if self.errors:
            IStatusMessage(self.request).addStatusMessage(_(u"There were errors."), "error")
            return

        purger = getUtility(IPurger)

        serverURL = self.request['SERVER_URL']

        def purge(url):
            if sync:
                status, xcache, xerror = purger.purgeSync(url)

                log = url
                if xcache:
                    log += " (X-Cache header: " + xcache + ")"
                if xerror:
                    log += " -- " + xerror
                self.purgeLog.append(log)
            else:
                purger.purgeAsync(url)
                self.purgeLog.append(url)

        portal_url = getToolByName(self.context, 'portal_url')
        portal = portal_url.getPortalObject()
        portalPath = portal.getPhysicalPath()

        proxies = self.purgingSettings.cachingProxies

        for inputURL in urls:
            if not inputURL.startswith(serverURL):  # not in the site
                if '://' in inputURL:  # Full URL?
                    purge(inputURL)
                else:                  # Path?
                    for newURL in getURLsToPurge(inputURL, proxies):
                        purge(newURL)
                continue

            physicalPath = relativePath = None
            try:
                physicalPath = self.request.physicalPathFromURL(inputURL)
            except ValueError:
                purge(inputURL)
                continue

            if not physicalPath:
                purge(inputURL)
                continue

            relativePath = physicalPath[len(portalPath):]
            if not relativePath:
                purge(inputURL)
                continue

            obj = portal.unrestrictedTraverse(relativePath, None)
            if obj is None:
                purge(inputURL)
                continue

            for path in getPathsToPurge(obj, self.request):
                for newURL in getURLsToPurge(path, proxies):
                    purge(newURL)


class RAMCache(BaseView):
    """The RAM cache control panel
    """

    def update(self):
        if super(RAMCache, self).update():
            if 'form.button.Purge' in self.request.form:
                self.processPurge()

    def processPurge(self):

        if self.ramCache is None:
            IStatusMessage(self.request).addStatusMessage(_(u"RAM cache not installed."), "error")

        if self.errors:
            IStatusMessage(self.request).addStatusMessage(_(u"There were errors."), "error")
            return

        self.ramCache.invalidateAll()
        IStatusMessage(self.request).addStatusMessage(_(u"Cache purged."), "info")



from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import IntegrationTesting
from plone.app.testing import FunctionalTesting
from plone.app.testing import applyProfile

from zope.interface import implements

from zope.component import getUtility
from zope.component import provideUtility

from zope.configuration import xmlconfig

from plone.cachepurging.interfaces import IPurger

try:
    from plone.protect.authenticator import _getKeyring
except ImportError:
    # so we can run tests on plone 4.3
    from plone.keyring.interfaces import IKeyManager
    def _getKeyring(username):
        manager = getUtility(IKeyManager)
        return manager['_system']

import hmac
from hashlib import sha1 as sha


class FauxPurger(object):

    implements(IPurger)

    def __init__(self):
        self.reset()

    def reset(self):
        self._sync = []
        self._async = []

    def purgeAsync(self, url, httpVerb='PURGE'):
        self._async.append(url)

    def purgeSync(self, url, httpVerb='PURGE'):
        self._sync.append(url)

    def stopThreads(self, wait=False):
        pass

    errorHeaders = ('X-Squid-Error',)
    http_1_1 = True


class PloneAppCaching(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):

        # Load ZCML
        import plone.app.caching
        xmlconfig.file('configure.zcml', plone.app.caching, context=configurationContext)

        # Install fake purger
        self.oldPurger = getUtility(IPurger)
        provideUtility(FauxPurger(), IPurger)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'plone.app.caching:default')

        portal['portal_workflow'].setDefaultChain('simple_publication_workflow')

    def tearDownZope(self, app):
        # Store old purger
        provideUtility(self.oldPurger, IPurger)


PLONE_APP_CACHING_FIXTURE = PloneAppCaching()
PLONE_APP_CACHING_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_APP_CACHING_FIXTURE,), name="PloneAppCaching:Integration")
PLONE_APP_CACHING_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PLONE_APP_CACHING_FIXTURE,), name="PloneAppCaching:Functional")


def getToken(username):
    ring = _getKeyring(username)
    secret = ring.random()
    return hmac.new(secret, username, sha).hexdigest()



from setuptools import setup, find_packages

version = '1.2.6'

setup(name='plone.app.caching',
      version=version,
      description="Plone UI and default rules for plone.caching/z3c.caching",
      long_description=open("README.rst").read() + "\n" +
                       open("CHANGES.rst").read(),
      classifiers=[
          "Environment :: Web Environment",
          "Framework :: Plone",
          "Framework :: Zope2",
          "License :: OSI Approved :: GNU General Public License (GPL)",
          "Operating System :: OS Independent",
          "Programming Language :: Python",
        ],
      keywords='plone caching',
      author='Plone Foundation',
      author_email='plone-developers@lists.sourceforge.net',
      url='http://pypi.python.org/pypi/plone.app.caching',
      license='GPL version 2',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['plone', 'plone.app'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'python-dateutil',
          'plone.caching',
          'plone.cachepurging',
          'plone.app.registry >= 1.0b5',
          'zope.browserresource',
          'zope.interface',
          'zope.component',
          'zope.publisher',
          'zope.pagetemplate',
          'plone.memoize',
          'plone.protect',
          'plone.registry >= 1.0b4',
          'Products.CMFDynamicViewFTI',
          'Products.GenericSetup',
          'Products.CMFCore',
          'Products.statusmessages',
          'Zope2',
          'Acquisition',
          'plone.app.z3cform',
          'z3c.form',
          'z3c.zcmlhook',
      ],
      extras_require={
          'test': [
              'plone.app.contenttypes[test]',
              'plone.app.testing',
          ]
      },
      entry_points="""
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



import unittest2 as unittest
from plone.testing.zca import UNIT_TESTING

from zope.interface import implements

from zope.component import getUtility
from zope.component import adapter
from zope.component import adapts
from zope.component import provideHandler
from zope.component import provideUtility
from zope.component import provideAdapter

from zope.component.event import objectEventNotify

from zope.event import notify

from zope.lifecycleevent import ObjectModifiedEvent
from zope.lifecycleevent import ObjectAddedEvent
from zope.lifecycleevent import ObjectRemovedEvent
from zope.lifecycleevent import ObjectMovedEvent

from zope.globalrequest import setRequest

from plone.registry.interfaces import IRegistry
from plone.registry.fieldfactory import persistentFieldAdapter
from plone.registry import Registry

from z3c.caching.interfaces import IPurgeEvent
from z3c.caching.interfaces import IPurgePaths

from plone.app.caching.interfaces import IPloneCacheSettings

from Acquisition import Explicit, aq_base
from Products.CMFDynamicViewFTI.interfaces import IBrowserDefault
from Products.CMFCore.interfaces import IContentish
from Products.CMFCore.interfaces import IDiscussionResponse
from Products.Archetypes import atapi
from Products.Archetypes.Schema.factory import instanceSchemaFactory

from plone.app.caching.purge import purgeOnModified
from plone.app.caching.purge import purgeOnMovedOrRemoved

from plone.app.caching.purge import ContentPurgePaths
from plone.app.caching.purge import DiscussionItemPurgePaths
from plone.app.caching.purge import ObjectFieldPurgePaths

class Handler(object):

    def __init__(self):
        self.invocations = []

    @adapter(IPurgeEvent)
    def handler(self, event):
        self.invocations.append(event)

class FauxRequest(dict):
    pass

class FauxNonContent(Explicit):
    implements(IContentish)

    def __init__(self, name=None):
        self.__name__ = name
        self.__parent__ = None  # may be overridden by acquisition

    def getId(self):
        return self.__name__

    def virtual_url_path(self):
        parent = aq_base(self.__parent__)
        if parent is not None:
            return parent.virtual_url_path() + '/' + self.__name__
        else:
            return self.__name__

    def getPhysicalPath(self):
        return ('', )

class FauxContent(FauxNonContent):
    implements(IBrowserDefault)

    portal_type = 'testtype'

    def defaultView(self):
        return 'default-view'

class FauxDiscussable(Explicit):
    implements(IDiscussionResponse)

class TestPurgeRedispatch(unittest.TestCase):

    layer = UNIT_TESTING

    def setUp(self):
        self.handler = Handler()
        provideHandler(self.handler.handler)

        provideHandler(objectEventNotify)
        provideHandler(purgeOnModified)
        provideHandler(purgeOnMovedOrRemoved)

        provideAdapter(persistentFieldAdapter)
        provideUtility(Registry(), IRegistry)
        registry = getUtility(IRegistry)
        registry.registerInterface(IPloneCacheSettings)

        ploneSettings = registry.forInterface(IPloneCacheSettings)
        ploneSettings.purgedContentTypes = ('testtype',)

    def test_not_purged(self):
        context = FauxNonContent('new').__of__(FauxContent())

        notify(ObjectModifiedEvent(context))
        notify(ObjectAddedEvent(context))
        notify(ObjectRemovedEvent(context))

        self.assertEqual(0, len(self.handler.invocations))

    def test_modified(self):
        context = FauxContent()

        notify(ObjectModifiedEvent(context))

        self.assertEqual(1, len(self.handler.invocations))
        self.assertEqual(context, self.handler.invocations[0].object)

    def test_added(self):
        context = FauxContent('new').__of__(FauxContent())

        notify(ObjectAddedEvent(context, context.__parent__, 'new'))

        self.assertEqual(0, len(self.handler.invocations))

    def test_moved(self):
        context = FauxContent('new').__of__(FauxContent())

        notify(ObjectMovedEvent(context, FauxContent(), 'old',
                                context.__parent__, 'new'))

        self.assertEqual(1, len(self.handler.invocations))
        self.assertEqual(context, self.handler.invocations[0].object)

    def test_renamed(self):
        context = FauxContent('new').__of__(FauxContent())

        notify(ObjectMovedEvent(context,
                                context.__parent__, 'old',
                                context.__parent__, 'new'))

        self.assertEqual(1, len(self.handler.invocations))
        self.assertEqual(context, self.handler.invocations[0].object)

    def test_removed(self):
        context = FauxContent('new').__of__(FauxContent())

        notify(ObjectRemovedEvent(context, context.__parent__, 'new'))

        self.assertEqual(1, len(self.handler.invocations))
        self.assertEqual(context, self.handler.invocations[0].object)

class TestContentPurgePaths(unittest.TestCase):

    layer = UNIT_TESTING

    def test_no_default_view(self):
        context = FauxNonContent('foo')
        purger = ContentPurgePaths(context)

        self.assertEqual(['/foo/', '/foo/view'],
                         list(purger.getRelativePaths()))
        self.assertEqual([], list(purger.getAbsolutePaths()))

    def test_default_view(self):
        context = FauxContent('foo')
        purger = ContentPurgePaths(context)

        self.assertEqual(['/foo/', '/foo/view', '/foo/default-view'],
                         list(purger.getRelativePaths()))
        self.assertEqual([], list(purger.getAbsolutePaths()))

    def test_parent_not_default_view(self):
        context = FauxContent('foo').__of__(FauxContent('bar'))
        purger = ContentPurgePaths(context)

        self.assertEqual(['/bar/foo/', '/bar/foo/view', '/bar/foo/default-view'],
                         list(purger.getRelativePaths()))
        self.assertEqual([], list(purger.getAbsolutePaths()))

    def test_parent_default_view(self):
        context = FauxContent('default-view').__of__(FauxContent('bar'))
        purger = ContentPurgePaths(context)

        self.assertEqual(['/bar/default-view/', '/bar/default-view/view',
                          '/bar/default-view/default-view', '/bar', '/bar/', '/bar/view'],
                         list(purger.getRelativePaths()))
        self.assertEqual([], list(purger.getAbsolutePaths()))

class TestDiscussionItemPurgePaths(unittest.TestCase):

    layer = UNIT_TESTING

    def setUp(self):

        class FauxContentPurgePaths(object):
            implements(IPurgePaths)
            adapts(FauxContent)

            def __init__(self, context):
                self.context = context

            def getRelativePaths(self):
                return ['/' + self.context.virtual_url_path()]

            def getAbsolutePaths(self):
                return ['/purgeme']

        provideAdapter(FauxContentPurgePaths, name="testpurge")

    def test_no_tool(self):
        root = FauxContent('')
        content = FauxContent('foo').__of__(root)
        discussable = FauxDiscussable().__of__(content)

        request = FauxRequest()
        setRequest(request)

        purge = DiscussionItemPurgePaths(discussable)

        self.assertEqual([], list(purge.getRelativePaths()))
        self.assertEqual([], list(purge.getAbsolutePaths()))

    def test_no_request(self):
        root = FauxContent('app')
        content = FauxContent('foo').__of__(root)
        discussable = FauxDiscussable().__of__(content)

        class FauxPloneTool(object):

            def getDiscussionThread(self, item):
                return [content, item]

        root.plone_utils = FauxPloneTool()

        setRequest(None)

        purge = DiscussionItemPurgePaths(discussable)

        self.assertEqual([], list(purge.getRelativePaths()))
        self.assertEqual([], list(purge.getAbsolutePaths()))

    def test_no_discussion_thread(self):
        root = FauxContent('app')
        content = FauxContent('foo').__of__(root)
        discussable = FauxDiscussable().__of__(content)

        class FauxPloneTool(object):

            def getDiscussionThread(self, item):
                return []

        root.plone_utils = FauxPloneTool()

        request = FauxRequest()
        setRequest(request)

        purge = DiscussionItemPurgePaths(discussable)

        self.assertEqual([], list(purge.getRelativePaths()))
        self.assertEqual([], list(purge.getAbsolutePaths()))

    def test_paths_of_root(self):
        root = FauxContent('app')
        content = FauxContent('foo').__of__(root)
        discussable = FauxDiscussable().__of__(content)

        class FauxPloneTool(object):

            def getDiscussionThread(self, item):
                return [content, item]

        root.plone_utils = FauxPloneTool()

        request = FauxRequest()
        setRequest(request)

        purge = DiscussionItemPurgePaths(discussable)

        self.assertEqual(['/app/foo'], list(purge.getRelativePaths()))
        self.assertEqual(['/purgeme'], list(purge.getAbsolutePaths()))

class TestObjectFieldPurgePaths(unittest.TestCase):

    maxDiff = None
    layer = UNIT_TESTING

    def setUp(self):
        provideAdapter(instanceSchemaFactory)

    def test_no_file_image_fields(self):

        class ATNoFields(atapi.BaseContent):
            schema = atapi.Schema((atapi.StringField('foo'),))

        context = ATNoFields('foo')
        purger = ObjectFieldPurgePaths(context)

        self.assertEqual([], list(purger.getRelativePaths()))
        self.assertEqual([], list(purger.getAbsolutePaths()))

    def test_file_image_fields(self):
        from plone.app.blob.field import BlobField

        class ATMultipleFields(atapi.BaseContent):
            schema = atapi.Schema((
                atapi.StringField('foo'),
                atapi.FileField('file1'),
                atapi.ImageField('image1'),
                atapi.ImageField('image2', sizes={'mini': (50, 50), 'normal': (100, 100)}),
                BlobField('blob1'),
            ))

        root = FauxContent('')
        context = ATMultipleFields('foo').__of__(root)
        purger = ObjectFieldPurgePaths(context)

        self.assertEqual(['/foo/download', '/foo/at_download',
                          '/foo/at_download/file1', '/foo/file1',
                          '/foo/at_download/image1', '/foo/image1', '/foo/image1_thumb',
                          '/foo/at_download/image2', '/foo/image2', '/foo/image2_mini',
                          '/foo/image2_normal', '/foo/at_download/blob1', '/foo/blob1'],
                         list(purger.getRelativePaths()))
        self.assertEqual([], list(purger.getAbsolutePaths()))

    def test_file_image_text_fields(self):

        class ATMultipleFields(atapi.BaseContent):
            schema = atapi.Schema((
                atapi.StringField('foo'),
                atapi.FileField('file1'),
                atapi.ImageField('image1'),
                atapi.ImageField('image2', sizes={'mini': (50, 50), 'normal': (100, 100)}),
                atapi.TextField('text'),
            ))

        root = FauxContent('')
        context = ATMultipleFields('foo').__of__(root)
        purger = ObjectFieldPurgePaths(context)

        self.assertEqual(['/foo/download', '/foo/at_download',
                          '/foo/at_download/file1', '/foo/file1',
                          '/foo/at_download/image1', '/foo/image1', '/foo/image1_thumb',
                          '/foo/at_download/image2', '/foo/image2', '/foo/image2_mini',
                          '/foo/image2_normal'],
                         list(purger.getRelativePaths()))
        self.assertEqual([], list(purger.getAbsolutePaths()))



import unittest2 as unittest
from plone.testing.zca import UNIT_TESTING

import time
import datetime
import dateutil.parser
import dateutil.tz
import wsgiref.handlers
from StringIO import StringIO

from zope.interface import implements
from zope.interface import Interface
from zope.interface import classImplements
from zope.interface import alsoProvides

from zope.component import provideAdapter
from zope.component import provideUtility
from zope.component import adapts

from plone.memoize.interfaces import ICacheChooser

from z3c.caching.interfaces import ILastModified

from zope.annotation.interfaces import IAnnotations
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.annotation.attribute import AttributeAnnotations

from ZPublisher.HTTPRequest import HTTPRequest
from ZPublisher.HTTPRequest import HTTPResponse

from OFS.SimpleItem import SimpleItem

from Products.CMFCore.interfaces import IContentish

class DummyPublished(object):

    def __init__(self, parent=None):
        self.__parent__ = parent

def normalize_response_cache(value):
    # Zope < 2.13 incorrectly includes the HTTP status as a header;
    # Zope 2.13 does not
    if isinstance(value, tuple) and 'status' in value[1]:
        del value[1]['status']
    return value


class ResponseModificationHelpersTest(unittest.TestCase):

    layer = UNIT_TESTING

    def setUp(self):
        provideAdapter(AttributeAnnotations)
        classImplements(HTTPRequest, IAttributeAnnotatable)

    # doNotCache()

    def test_doNotCache(self):
        from plone.app.caching.operations.utils import doNotCache

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        now = datetime.datetime.now(dateutil.tz.tzlocal())

        doNotCache(published, request, response)

        self.assertEqual(200, response.getStatus())
        self.assertEqual('max-age=0, must-revalidate, private', response.getHeader('Cache-Control'))
        self.assertEqual(None, response.getHeader('Last-Modified'))

        expires = dateutil.parser.parse(response.getHeader('Expires'))
        self.assertTrue(now > expires)

    def test_doNotCache_deletes_last_modified(self):
        from plone.app.caching.operations.utils import doNotCache

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        now = datetime.datetime.now(dateutil.tz.tzlocal())
        response.setHeader('Last-Modified', wsgiref.handlers.format_date_time(time.mktime(now.timetuple())))

        doNotCache(published, request, response)

        self.assertEqual(200, response.getStatus())
        self.assertEqual('max-age=0, must-revalidate, private', response.getHeader('Cache-Control'))
        self.assertEqual(None, response.getHeader('Last-Modified'))

        expires = dateutil.parser.parse(response.getHeader('Expires'))
        self.assertTrue(now > expires)


    # cacheInBrowser()

    def test_cacheInBrowser_no_etag_no_last_modified(self):
        from plone.app.caching.operations.utils import cacheInBrowser

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        now = datetime.datetime.now(dateutil.tz.tzlocal())

        cacheInBrowser(published, request, response)

        self.assertEqual(200, response.getStatus())
        self.assertEqual('max-age=0, must-revalidate, private', response.getHeader('Cache-Control'))
        self.assertEqual(None, response.getHeader('Last-Modified'))
        self.assertEqual(None, response.getHeader('ETag', literal=1))

        expires = dateutil.parser.parse(response.getHeader('Expires'))
        self.assertTrue(now > expires)

    def test_cacheInBrowser_etag(self):
        from plone.app.caching.operations.utils import cacheInBrowser

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        now = datetime.datetime.now(dateutil.tz.tzlocal())
        etag = "|foo|bar|"

        cacheInBrowser(published, request, response, etag=etag)

        self.assertEqual(200, response.getStatus())
        self.assertEqual('max-age=0, must-revalidate, private', response.getHeader('Cache-Control'))
        self.assertEqual(None, response.getHeader('Last-Modified'))
        self.assertEqual('"|foo|bar|"', response.getHeader('ETag', literal=1))

        expires = dateutil.parser.parse(response.getHeader('Expires'))
        self.assertTrue(now > expires)

    def test_cacheInBrowser_lastModified(self):
        from plone.app.caching.operations.utils import cacheInBrowser

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        now = datetime.datetime.now(dateutil.tz.tzlocal())
        nowFormatted = wsgiref.handlers.format_date_time(time.mktime(now.timetuple()))

        cacheInBrowser(published, request, response, lastModified=now)

        self.assertEqual(200, response.getStatus())
        self.assertEqual('max-age=0, must-revalidate, private', response.getHeader('Cache-Control'))
        self.assertEqual(nowFormatted, response.getHeader('Last-Modified'))
        self.assertEqual(None, response.getHeader('ETag', literal=1))

        expires = dateutil.parser.parse(response.getHeader('Expires'))
        self.assertTrue(now > expires)

    def test_cacheInBrowser_lastModified_and_etag(self):
        from plone.app.caching.operations.utils import cacheInBrowser

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        now = datetime.datetime.now(dateutil.tz.tzlocal())
        etag = "|foo|bar|"

        nowFormatted = wsgiref.handlers.format_date_time(time.mktime(now.timetuple()))

        cacheInBrowser(published, request, response, etag=etag, lastModified=now)

        self.assertEqual(200, response.getStatus())
        self.assertEqual('max-age=0, must-revalidate, private', response.getHeader('Cache-Control'))
        self.assertEqual(nowFormatted, response.getHeader('Last-Modified'))
        self.assertEqual('"|foo|bar|"', response.getHeader('ETag', literal=1))

        expires = dateutil.parser.parse(response.getHeader('Expires'))
        self.assertTrue(now > expires)


    # cacheInProxy()

    def test_cacheInProxy_minimal(self):
        from plone.app.caching.operations.utils import cacheInProxy

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        now = datetime.datetime.now(dateutil.tz.tzlocal())

        cacheInProxy(published, request, response, smaxage=60)

        self.assertEqual(200, response.getStatus())
        self.assertEqual('max-age=0, s-maxage=60, must-revalidate', response.getHeader('Cache-Control'))
        self.assertEqual(None, response.getHeader('Last-Modified'))
        self.assertEqual(None, response.getHeader('ETag', literal=1))
        self.assertEqual(None, response.getHeader('Vary'))

        expires = dateutil.parser.parse(response.getHeader('Expires'))
        self.assertTrue(now > expires)

    def test_cacheInProxy_full(self):
        from plone.app.caching.operations.utils import cacheInProxy

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        now = datetime.datetime.now(dateutil.tz.tzlocal())
        etag = '|foo|bar|'
        vary = 'Accept-Language'

        nowFormatted = wsgiref.handlers.format_date_time(time.mktime(now.timetuple()))

        cacheInProxy(published, request, response, smaxage=60, etag=etag, lastModified=now, vary=vary)

        self.assertEqual(200, response.getStatus())
        self.assertEqual('max-age=0, s-maxage=60, must-revalidate', response.getHeader('Cache-Control'))
        self.assertEqual(nowFormatted, response.getHeader('Last-Modified'))
        self.assertEqual('"|foo|bar|"', response.getHeader('ETag', literal=1))
        self.assertEqual('Accept-Language', response.getHeader('Vary'))

        expires = dateutil.parser.parse(response.getHeader('Expires'))
        self.assertTrue(now > expires)


    # cacheInBrowserAndProxy()

    def test_cacheInBrowserAndProxy_minimal(self):
        from plone.app.caching.operations.utils import cacheInBrowserAndProxy

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        now = datetime.datetime.now(dateutil.tz.tzlocal())

        cacheInBrowserAndProxy(published, request, response, maxage=60)

        self.assertEqual(200, response.getStatus())
        self.assertEqual('max-age=60, proxy-revalidate, public', response.getHeader('Cache-Control'))
        self.assertEqual(None, response.getHeader('Last-Modified'))
        self.assertEqual(None, response.getHeader('ETag', literal=1))
        self.assertEqual(None, response.getHeader('Vary'))

        timedelta = dateutil.parser.parse(response.getHeader('Expires')) - now
        self.assertFalse(timedelta < datetime.timedelta(seconds=59))
        self.assertFalse(timedelta > datetime.timedelta(seconds=60))

    def test_cacheInBrowserAndProxy_full(self):
        from plone.app.caching.operations.utils import cacheInBrowserAndProxy

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        now = datetime.datetime.now(dateutil.tz.tzlocal())
        etag = '|foo|bar|'
        vary = 'Accept-Language'

        nowFormatted = wsgiref.handlers.format_date_time(time.mktime(now.timetuple()))

        cacheInBrowserAndProxy(published, request, response, maxage=60, etag=etag, lastModified=now, vary=vary)

        self.assertEqual(200, response.getStatus())
        self.assertEqual('max-age=60, proxy-revalidate, public', response.getHeader('Cache-Control'))
        self.assertEqual(nowFormatted, response.getHeader('Last-Modified'))
        self.assertEqual('"|foo|bar|"', response.getHeader('ETag', literal=1))
        self.assertEqual('Accept-Language', response.getHeader('Vary'))

        timedelta = dateutil.parser.parse(response.getHeader('Expires')) - now
        self.assertFalse(
            timedelta < datetime.timedelta(seconds=58),
            "%s is not < %s" % (timedelta, datetime.timedelta(seconds=58))
        )
        self.assertFalse(
            timedelta > datetime.timedelta(seconds=61),
            "%s is not > %s" % (timedelta, datetime.timedelta(seconds=61))
        )


    # cacheInRAM()

    def test_cacheInRAM_no_etag(self):
        from plone.app.caching.operations.utils import cacheInRAM
        from plone.app.caching.operations.utils import PAGE_CACHE_ANNOTATION_KEY

        from plone.app.caching.interfaces import IRAMCached

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        request.environ['PATH_INFO'] = '/foo'

        assert not IRAMCached.providedBy(response)

        cacheInRAM(published, request, response)

        annotations = IAnnotations(request)
        self.assertEqual("http://example.com/foo?", annotations[PAGE_CACHE_ANNOTATION_KEY])
        self.assertTrue(IRAMCached.providedBy(request))

    def test_cacheInRAM_etag(self):
        from plone.app.caching.operations.utils import cacheInRAM
        from plone.app.caching.operations.utils import PAGE_CACHE_ANNOTATION_KEY

        from plone.app.caching.interfaces import IRAMCached

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        request.environ['PATH_INFO'] = '/foo'
        etag = '|foo|bar|'

        assert not IRAMCached.providedBy(response)

        cacheInRAM(published, request, response, etag=etag)

        annotations = IAnnotations(request)
        self.assertEqual("||foo|bar|||http://example.com/foo?", annotations[PAGE_CACHE_ANNOTATION_KEY])
        self.assertTrue(IRAMCached.providedBy(request))

    def test_cacheInRAM_etag_alternate_key(self):
        from plone.app.caching.operations.utils import cacheInRAM

        from plone.app.caching.interfaces import IRAMCached

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        request.environ['PATH_INFO'] = '/foo'
        etag = '|foo|bar|'

        assert not IRAMCached.providedBy(response)

        cacheInRAM(published, request, response, etag=etag, annotationsKey="alt.key")

        annotations = IAnnotations(request)
        self.assertEqual("||foo|bar|||http://example.com/foo?", annotations["alt.key"])
        self.assertTrue(IRAMCached.providedBy(request))


class ResponseInterceptorHelpersTest(unittest.TestCase):

    layer = UNIT_TESTING

    def setUp(self):
        provideAdapter(AttributeAnnotations)
        classImplements(HTTPRequest, IAttributeAnnotatable)

    # cachedResponse()

    def test_cachedResponse(self):
        from plone.app.caching.operations.utils import cachedResponse

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        headers = {
            'X-Cache-Rule': 'foo',
            'X-Foo': 'bar',
            'ETag': '||blah||',
        }

        response.setHeader('X-Foo', 'baz')
        response.setHeader('X-Bar', 'qux')
        response.setStatus(200)

        body = cachedResponse(published, request, response, 404, headers, u"body")

        self.assertEqual(u"body", body)
        self.assertEqual(404, response.getStatus())
        self.assertEqual('foo', response.getHeader('X-Cache-Rule'))
        self.assertEqual('bar', response.getHeader('X-Foo'))
        self.assertEqual('qux', response.getHeader('X-Bar'))
        self.assertEqual('||blah||', response.getHeader('ETag', literal=1))

    def test_cachedResponse_gzip_off(self):
        from plone.app.caching.operations.utils import cachedResponse

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        headers = {
            'X-Cache-Rule': 'foo',
            'X-Foo': 'bar',
            'ETag': '||blah||',
        }

        response.setHeader('X-Foo', 'baz')
        response.setHeader('X-Bar', 'qux')
        response.setStatus(200)

        request.environ['HTTP_ACCEPT_ENCODING'] = 'gzip; deflate'
        response.enableHTTPCompression(request)

        assert response.enableHTTPCompression(query=True)

        body = cachedResponse(published, request, response, 404, headers, u"body", 0)

        self.assertEqual(u"body", body)
        self.assertEqual(404, response.getStatus())
        self.assertEqual('foo', response.getHeader('X-Cache-Rule'))
        self.assertEqual('bar', response.getHeader('X-Foo'))
        self.assertEqual('qux', response.getHeader('X-Bar'))
        self.assertEqual('||blah||', response.getHeader('ETag', literal=1))

        self.assertFalse(response.enableHTTPCompression(query=True))

    def test_cachedResponse_gzip_on(self):
        from plone.app.caching.operations.utils import cachedResponse

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        headers = {
            'X-Cache-Rule': 'foo',
            'X-Foo': 'bar',
            'ETag': '||blah||',
        }

        response.setHeader('X-Foo', 'baz')
        response.setHeader('X-Bar', 'qux')
        response.setStatus(200)

        request.environ['HTTP_ACCEPT_ENCODING'] = 'gzip; deflate'
        response.enableHTTPCompression(request, disable=True)

        assert not response.enableHTTPCompression(query=True)

        body = cachedResponse(published, request, response, 404, headers, u"body", 1)

        self.assertEqual(u"body", body)
        self.assertEqual(404, response.getStatus())
        self.assertEqual('foo', response.getHeader('X-Cache-Rule'))
        self.assertEqual('bar', response.getHeader('X-Foo'))
        self.assertEqual('qux', response.getHeader('X-Bar'))
        self.assertEqual('||blah||', response.getHeader('ETag', literal=1))

        self.assertTrue(response.enableHTTPCompression(query=True))

    # notModified()

    def test_notModified_minimal(self):
        from plone.app.caching.operations.utils import notModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        response.setStatus(200)

        body = notModified(published, request, response)

        self.assertEqual(u"", body)
        self.assertEqual(304, response.getStatus())

    def test_notModified_full(self):
        from plone.app.caching.operations.utils import notModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        response.setStatus(200)

        now = datetime.datetime.now(dateutil.tz.tzlocal())
        etag = "|foo|bar|"

        body = notModified(published, request, response, etag=etag, lastModified=now)

        self.assertEqual(u"", body)
        self.assertEqual(etag, response.getHeader('ETag', literal=1))
        self.assertEqual(None, response.getHeader('Last-Modified'))
        self.assertEqual(None, response.getHeader('Expires'))
        self.assertEqual(None, response.getHeader('Cache-Control'))
        self.assertEqual(304, response.getStatus())

class CacheCheckHelpersTest(unittest.TestCase):

    layer = UNIT_TESTING

    def setUp(self):
        provideAdapter(AttributeAnnotations)
        classImplements(HTTPRequest, IAttributeAnnotatable)

    # isModified()

    def test_isModified_no_headers_no_keys(self):
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        self.assertEqual(True, isModified(request))

    def test_isModified_no_headers_with_keys(self):
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        etag = '|foo'
        lastModified = datetime.datetime(2010, 11, 24, 3, 4, 5, 6, dateutil.tz.tzutc())

        self.assertEqual(True, isModified(request, etag=etag, lastModified=lastModified))

    def test_isModified_ims_invalid_date(self):
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['HTTP_IF_MODIFIED_SINCE'] = 'blah'

        lastModified = datetime.datetime(2010, 11, 24, 3, 4, 5, 6, dateutil.tz.tzutc())

        self.assertEqual(True, isModified(request, lastModified=lastModified))

    def test_isModified_ims_modified(self):
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['HTTP_IF_MODIFIED_SINCE'] = 'Tue, 24 Nov 2009 03:04:05 GMT'

        lastModified = datetime.datetime(2010, 11, 24, 3, 4, 5, 6, dateutil.tz.tzutc())

        self.assertEqual(True, isModified(request, lastModified=lastModified))

    def test_isModified_ims_not_modified(self):
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['HTTP_IF_MODIFIED_SINCE'] = 'Thu, 24 Nov 2011 03:04:05 GMT'

        lastModified = datetime.datetime(2010, 11, 24, 3, 4, 5, 6, dateutil.tz.tzutc())

        self.assertEqual(False, isModified(request, lastModified=lastModified))

    def test_isModified_ims_not_modified_two_dates(self):
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['HTTP_IF_MODIFIED_SINCE'] = 'Thu, 24 Nov 2011 03:04:05 GMT; Thu, 24 Nov 2011 03:04:05'

        lastModified = datetime.datetime(2010, 11, 24, 3, 4, 5, 6, dateutil.tz.tzutc())

        self.assertEqual(False, isModified(request, lastModified=lastModified))

    def test_isModified_ims_not_modified_etag_no_inm_header(self):
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['HTTP_IF_MODIFIED_SINCE'] = 'Thu, 24 Nov 2011 03:04:05 GMT'

        etag = '|foo'
        lastModified = datetime.datetime(2010, 11, 24, 3, 4, 5, 6, dateutil.tz.tzutc())

        self.assertEqual(True, isModified(request, etag=etag, lastModified=lastModified))

    def test_isModified_inm_no_tags(self):
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        etag = '|foo'

        self.assertEqual(True, isModified(request, etag=etag))

    def test_isModified_inm_one_tag_no_match(self):
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['HTTP_IF_NONE_MATCH'] = '"|bar"'

        etag = '|foo'

        self.assertEqual(True, isModified(request, etag=etag))

    def test_isModified_inm_multiple_tags_no_match(self):
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['HTTP_IF_NONE_MATCH'] = '"|bar", W/"|baz"'

        etag = '|foo'

        self.assertEqual(True, isModified(request, etag=etag))

    def test_isModified_inm_invalid_tag(self):
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['HTTP_IF_NONE_MATCH'] = '"'

        etag = '|foo'

        self.assertEqual(True, isModified(request, etag=etag))

    def test_isModified_inm_star(self):
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['HTTP_IF_NONE_MATCH'] = '"*", W/"|baz"'

        etag = '|foo'

        self.assertEqual(False, isModified(request, etag=etag))

    def test_isModified_inm_match_single(self):
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['HTTP_IF_NONE_MATCH'] = '"|foo"'

        etag = '|foo'

        self.assertEqual(False, isModified(request, etag=etag))

    def test_isModified_inm_match_update(self):
        """
        If a site was previously configured to use etags and then the
        configuration updates, cached pages from the previous config
        will send and If-None-Match but will have no site configuration
        for etags. In this case, modified should return True since the
        headers need to be updated with the new config. Additionally,
        last modified may be attached to the request header, but
        If-None-Match always takes precedence over these headers and
        therefor they should be ignored.
        """
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        # at some point the code added dummy timestamps to this header
        request.environ['HTTP_IF_NONE_MATCH'] = '"1232342354.65"'

        etag = None

        self.assertEqual(True, isModified(request, etag=etag,
                                           lastModified='doesnt_really_matter'))


    def test_isModified_inm_match_multiple(self):
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['HTTP_IF_NONE_MATCH'] = '"|foo", "|bar"'

        etag = '|foo'

        self.assertEqual(False, isModified(request, etag=etag))

    def test_isModified_inm_match_weak(self):
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['HTTP_IF_NONE_MATCH'] = 'W/"|foo"'

        etag = '|foo'

        self.assertEqual(False, isModified(request, etag=etag))

    def test_isModified_inm_match_ignores_ims_if_no_last_modified_date(self):
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['HTTP_IF_NONE_MATCH'] = '"|foo"'
        request.environ['HTTP_IF_MODIFIED_SINCE'] = 'Tue, 24 Nov 2009 03:04:05 GMT'

        etag = '|foo'

        self.assertEqual(False, isModified(request, etag=etag))

    def test_isModified_inm_match_modified(self):
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['HTTP_IF_NONE_MATCH'] = '"|foo"'
        request.environ['HTTP_IF_MODIFIED_SINCE'] = 'Tue, 24 Nov 2009 03:04:05 GMT'

        etag = '|foo'
        lastModified = datetime.datetime(2010, 11, 24, 3, 4, 5, 6, dateutil.tz.tzutc())

        self.assertEqual(True, isModified(request, etag=etag, lastModified=lastModified))

    def test_isModified_inm_match_not_modified(self):
        from plone.app.caching.operations.utils import isModified

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['HTTP_IF_NONE_MATCH'] = '"|foo"'
        request.environ['HTTP_IF_MODIFIED_SINCE'] = 'Thu, 24 Nov 2011 03:04:05 GMT'

        etag = '|foo'
        lastModified = datetime.datetime(2010, 11, 24, 3, 4, 5, 6, dateutil.tz.tzutc())

        self.assertEqual(False, isModified(request, etag=etag, lastModified=lastModified))

    # visibleToRole()

    def test_visibleToRole_not_real(self):
        from plone.app.caching.operations.utils import visibleToRole
        published = DummyPublished()
        self.assertEqual(False, visibleToRole(published, role='Anonymous'))

    def test_visibleToRole_permission(self):
        from plone.app.caching.operations.utils import visibleToRole

        s = SimpleItem()

        s.manage_permission('View', ('Member', 'Manager',))
        self.assertEqual(False, visibleToRole(s, role='Anonymous'))

        s.manage_permission('View', ('Member', 'Manager', 'Anonymous',))
        self.assertEqual(True, visibleToRole(s, role='Anonymous'))


class MiscHelpersTest(unittest.TestCase):

    layer = UNIT_TESTING

    def setUp(self):
        provideAdapter(AttributeAnnotations)
        classImplements(HTTPRequest, IAttributeAnnotatable)

    # getContext()

    def test_getContext(self):
        from plone.app.caching.operations.utils import getContext

        class Parent(object):
            implements(IContentish)

        parent = Parent()
        published = DummyPublished(parent)

        self.assertTrue(getContext(parent) is parent)
        self.assertTrue(getContext(published) is parent)

    def test_getContext_custom_marker(self):
        from plone.app.caching.operations.utils import getContext

        class Parent(object):
            implements(IContentish)

            def __init__(self, parent=None):
                self.__parent__ = parent

        class IDummy(Interface):
            pass

        grandparent = Parent()
        parent = Parent(grandparent)
        published = DummyPublished(parent)

        self.assertTrue(getContext(published, marker=IDummy) is None)
        self.assertTrue(getContext(published, marker=(IDummy,)) is None)

        alsoProvides(grandparent, IDummy)

        self.assertTrue(getContext(parent, marker=IDummy) is grandparent)
        self.assertTrue(getContext(published, marker=(IDummy,)) is grandparent)

    # formatDateTime()

    def test_formatDateTime_utc(self):
        from plone.app.caching.operations.utils import formatDateTime

        dt = datetime.datetime(2010, 11, 24, 3, 4, 5, 6, dateutil.tz.tzutc())
        self.assertEqual('Wed, 24 Nov 2010 03:04:05 GMT', formatDateTime(dt))

    def test_formatDateTime_local(self):
        from plone.app.caching.operations.utils import formatDateTime

        dt = datetime.datetime(2010, 11, 24, 3, 4, 5, 6, dateutil.tz.tzlocal())
        inGMT = formatDateTime(dt)

        # Who knows what your local timezone is :-)
        self.assertTrue(inGMT.endswith(' GMT'))
        self.assertTrue('Nov 2010' in inGMT)

        # We lose microseconds. Big whoop.
        p = dateutil.parser.parse(inGMT).astimezone(dateutil.tz.tzlocal())
        lofi = datetime.datetime(2010, 11, 24, 3, 4, 5, 0, dateutil.tz.tzlocal())
        self.assertEqual(p, lofi)

    def test_formatDateTime_naive(self):
        from plone.app.caching.operations.utils import formatDateTime

        dt = datetime.datetime(2010, 11, 24, 3, 4, 5, 6)
        inGMT = formatDateTime(dt)

        # Who knows what your local timezone is :-)
        self.assertTrue(inGMT.endswith(' GMT'))
        self.assertTrue('Nov 2010' in inGMT)

        # Can't compare offset aware and naive
        p = dateutil.parser.parse(inGMT).astimezone(dateutil.tz.tzlocal())
        self.assertEqual((2010, 11, 24, 3, 4, 5), (p.year, p.month, p.day, p.hour, p.minute, p.second))

    # parseDateTime()

    def test_parseDateTime_invalid(self):
        from plone.app.caching.operations.utils import parseDateTime

        self.assertEqual(None, parseDateTime("foo"))

    def test_parseDateTime_rfc1123(self):
        from plone.app.caching.operations.utils import parseDateTime

        dt = datetime.datetime(2010, 11, 23, 3, 4, 5, 0, dateutil.tz.tzutc())
        self.assertEqual(dt, parseDateTime("'Tue, 23 Nov 2010 3:04:05 GMT'"))

    def test_formatDateTime_no_timezone(self):
        from plone.app.caching.operations.utils import parseDateTime

        # parser will assume input was local time
        dt = datetime.datetime(2010, 11, 23, 19, 4, 5, 0, dateutil.tz.tzlocal())
        self.assertEqual(dt, parseDateTime("'Tue, 23 Nov 2010 19:04:05'"))

    # getLastModified()

    def test_getLastModified_no_adaper(self):
        from plone.app.caching.operations.utils import getLastModified

        published = DummyPublished()
        self.assertEqual(None, getLastModified(published))

    def test_getLastModified_none(self):
        from plone.app.caching.operations.utils import getLastModified

        class DummyLastModified(object):
            implements(ILastModified)
            adapts(DummyPublished)

            def __init__(self, context):
                self.context = context

            def __call__(self):
                return None

        provideAdapter(DummyLastModified)

        published = DummyPublished()
        self.assertEqual(None, getLastModified(published))

    def test_getLastModified_missing_timezone(self):
        from plone.app.caching.operations.utils import getLastModified

        class DummyLastModified(object):
            implements(ILastModified)
            adapts(DummyPublished)

            def __init__(self, context):
                self.context = context

            def __call__(self):
                return datetime.datetime(2010, 11, 24, 3, 4, 5, 6)

        provideAdapter(DummyLastModified)

        published = DummyPublished()
        self.assertEqual(datetime.datetime(2010, 11, 24, 3, 4, 5, 6, dateutil.tz.tzlocal()),
                          getLastModified(published))

    def test_getLastModified_timezone(self):
        from plone.app.caching.operations.utils import getLastModified

        class DummyLastModified(object):
            implements(ILastModified)
            adapts(DummyPublished)

            def __init__(self, context):
                self.context = context

            def __call__(self):
                return datetime.datetime(2010, 11, 24, 3, 4, 5, 6, dateutil.tz.tzutc())

        provideAdapter(DummyLastModified)

        published = DummyPublished()
        self.assertEqual(datetime.datetime(2010, 11, 24, 3, 4, 5, 6, dateutil.tz.tzutc()),
                          getLastModified(published))


    # getExpiration()

    def test_getExpiration_0(self):
        from plone.app.caching.operations.utils import getExpiration

        now = datetime.datetime.now()
        val = getExpiration(0)

        difference = now - val

        # it's more than a year in the past, which is plenty; it's actually
        # more like 10 years in the past, but it's hard to compare when the
        # calculation is based on the current time of the test.
        self.assertTrue(difference >= datetime.timedelta(days=365))

    def test_getExpiration_past(self):
        from plone.app.caching.operations.utils import getExpiration

        now = datetime.datetime.now()
        val = getExpiration(-1)

        difference = now - val

        # any value in the past basically has the same effect as setting -1
        self.assertTrue(difference >= datetime.timedelta(days=365))

    def test_getExpiration_future(self):
        from plone.app.caching.operations.utils import getExpiration

        now = datetime.datetime.now()
        val = getExpiration(60)

        difference = val - now

        # give the test two seconds' leeway
        self.assertTrue(difference >= datetime.timedelta(seconds=58))


    # getETag()

    def test_getETag_extra_only(self):
        from plone.app.caching.operations.utils import getETag

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        self.assertEqual('|foo|bar;baz', getETag(published, request, extraTokens=('foo', 'bar,baz')))

    def test_getETag_key_not_found(self):
        from plone.app.caching.operations.utils import getETag

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        self.assertEqual(None, getETag(published, request, keys=('foo', 'bar',)))

    def test_getETag_adapter_returns_none(self):
        from plone.app.caching.operations.utils import getETag
        from plone.app.caching.interfaces import IETagValue

        class FooETag(object):
            implements(IETagValue)
            adapts(DummyPublished, HTTPRequest)

            def __init__(self, published, request):
                self.published = published
                self.request = request

            def __call__(self):
                return 'foo'

        provideAdapter(FooETag, name=u"foo")

        class BarETag(object):
            implements(IETagValue)
            adapts(DummyPublished, HTTPRequest)

            def __init__(self, published, request):
                self.published = published
                self.request = request

            def __call__(self):
                return None

        provideAdapter(BarETag, name=u"bar")

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        self.assertEqual('|foo|', getETag(published, request, keys=('foo', 'bar',)))

    def test_getETag_full(self):
        from plone.app.caching.operations.utils import getETag
        from plone.app.caching.interfaces import IETagValue

        class FooETag(object):
            implements(IETagValue)
            adapts(DummyPublished, HTTPRequest)

            def __init__(self, published, request):
                self.published = published
                self.request = request

            def __call__(self):
                return 'foo'

        provideAdapter(FooETag, name=u"foo")

        class BarETag(object):
            implements(IETagValue)
            adapts(DummyPublished, HTTPRequest)

            def __init__(self, published, request):
                self.published = published
                self.request = request

            def __call__(self):
                return 'bar'

        provideAdapter(BarETag, name=u"bar")

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)
        published = DummyPublished()

        self.assertEqual('|foo|bar|baz;qux', getETag(published, request,
                keys=('foo', 'bar',), extraTokens=('baz,qux',)))


    # parseETags()

    def test_parseETags_empty(self):
        from plone.app.caching.operations.utils import parseETags
        self.assertEqual([], parseETags(''))

    def test_parseETags_None(self):
        from plone.app.caching.operations.utils import parseETags
        self.assertEqual([], parseETags(''))

    def test_parseETags_star(self):
        from plone.app.caching.operations.utils import parseETags
        self.assertEqual(['*'], parseETags('*'))

    def test_parseETags_star_quoted(self):
        from plone.app.caching.operations.utils import parseETags
        self.assertEqual(['*'], parseETags('"*"'))

    def test_parseETags_single(self):
        from plone.app.caching.operations.utils import parseETags
        self.assertEqual(['|foo|bar;baz'], parseETags('|foo|bar;baz'))

    def test_parseETags_single_quoted(self):
        from plone.app.caching.operations.utils import parseETags
        self.assertEqual(['|foo|bar;baz'], parseETags('"|foo|bar;baz"'))

    def test_parseETags_multiple(self):
        from plone.app.caching.operations.utils import parseETags
        self.assertEqual(['|foo|bar;baz', '1234'], parseETags('|foo|bar;baz, 1234'))

    def test_parseETags_multiple_quoted(self):
        from plone.app.caching.operations.utils import parseETags
        self.assertEqual(['|foo|bar;baz', '1234'], parseETags('"|foo|bar;baz", "1234"'))

    def test_parseETags_multiple_nospace(self):
        from plone.app.caching.operations.utils import parseETags
        self.assertEqual(['|foo|bar;baz', '1234'], parseETags('|foo|bar;baz,1234'))

    def test_parseETags_multiple_quoted_nospace(self):
        from plone.app.caching.operations.utils import parseETags
        self.assertEqual(['|foo|bar;baz', '1234'], parseETags('"|foo|bar;baz","1234"'))

    def test_parseETags_multiple_weak(self):
        from plone.app.caching.operations.utils import parseETags
        self.assertEqual(['|foo|bar;baz', '1234'], parseETags('|foo|bar;baz, W/1234'))

    def test_parseETags_multiple_quoted_weak(self):
        from plone.app.caching.operations.utils import parseETags
        self.assertEqual(['|foo|bar;baz', '1234'], parseETags('"|foo|bar;baz", W/"1234"'))

    def test_parseETags_multiple_weak_disallowed(self):
        from plone.app.caching.operations.utils import parseETags
        self.assertEqual(['|foo|bar;baz'], parseETags('|foo|bar;baz, W/1234', allowWeak=False))

    def test_parseETags_multiple_quoted_weak_disallowed(self):
        from plone.app.caching.operations.utils import parseETags
        self.assertEqual(['|foo|bar;baz'], parseETags('"|foo|bar;baz", W/"1234"', allowWeak=False))

class RAMCacheTest(unittest.TestCase):

    layer = UNIT_TESTING

    def setUp(self):
        provideAdapter(AttributeAnnotations)
        classImplements(HTTPRequest, IAttributeAnnotatable)

    # getRAMCache()

    def test_getRAMCache_no_chooser(self):
        from plone.app.caching.operations.utils import getRAMCache
        self.assertEqual(None, getRAMCache())

    def test_getRAMCache_custom_global_key(self):
        from plone.app.caching.operations.utils import getRAMCache

        class Cache(dict):
            pass

        cache = Cache()

        class Chooser(object):
            implements(ICacheChooser)

            def __call__(self, key):
                assert key == 'foo'
                return cache

        provideUtility(Chooser())
        self.assertEqual(cache, getRAMCache('foo'))

    def test_getRAMCache_normal(self):
        from plone.app.caching.operations.utils import getRAMCache

        class Cache(dict):
            pass

        cache = Cache()

        class Chooser(object):
            implements(ICacheChooser)

            def __call__(self, key):
                assert key == 'plone.app.caching.operations.ramcache'
                return cache

        provideUtility(Chooser())
        self.assertEqual(cache, getRAMCache())


    # getRAMCacheKey()

    def test_getRAMCacheKey_empty(self):
        from plone.app.caching.operations.utils import getRAMCacheKey

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        self.assertEqual('http://example.com?', getRAMCacheKey(request))

    def test_getRAMCacheKey_normal(self):
        from plone.app.caching.operations.utils import getRAMCacheKey

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['PATH_INFO'] = '/foo/bar'
        request.environ['QUERY_STRING'] = 'x=1&y=2'

        self.assertEqual('http://example.com/foo/bar?x=1&y=2', getRAMCacheKey(request))

    def test_getRAMCacheKey_etag(self):
        from plone.app.caching.operations.utils import getRAMCacheKey

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['PATH_INFO'] = '/foo/bar'
        request.environ['QUERY_STRING'] = 'x=1&y=2'

        self.assertEqual('||foo|bar||http://example.com/foo/bar?x=1&y=2', getRAMCacheKey(request, etag="|foo|bar"))


    # storeResponseInRAMCache()

    def test_storeResponseInRAMCache_no_key(self):
        from plone.app.caching.operations.utils import storeResponseInRAMCache

        class Cache(dict):
            pass

        cache = Cache()

        class Chooser(object):
            implements(ICacheChooser)

            def __call__(self, key):
                assert key == 'plone.app.caching.operations.ramcache'
                return cache

        provideUtility(Chooser())

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        result = u"Body"
        response.setHeader('X-Foo', 'bar')

        storeResponseInRAMCache(request, response, result)

        self.assertEqual(0, len(cache))

    def test_storeResponseInRAMCache_no_cache(self):
        from plone.app.caching.operations.utils import storeResponseInRAMCache

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        result = u"Body"
        response.setHeader('X-Foo', 'bar')

        IAnnotations(request)['plone.app.caching.operations.ramcache.key'] = 'foo'

        storeResponseInRAMCache(request, response, result)

    def test_storeResponseInRAMCache_normal(self):
        from plone.app.caching.operations.utils import storeResponseInRAMCache

        class Cache(dict):
            pass

        cache = Cache()

        class Chooser(object):
            implements(ICacheChooser)

            def __call__(self, key):
                assert key == 'plone.app.caching.operations.ramcache'
                return cache

        provideUtility(Chooser())

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        result = u"Body"
        response.setHeader('X-Foo', 'bar')

        IAnnotations(request)['plone.app.caching.operations.ramcache.key'] = 'foo'

        storeResponseInRAMCache(request, response, result)

        self.assertEqual(1, len(cache))
        cached = normalize_response_cache(cache['foo'])
        self.assertEqual((200, {'x-foo': 'bar'}, u'Body', 0), cached)

    def test_storeResponseInRAMCache_gzip(self):
        from plone.app.caching.operations.utils import storeResponseInRAMCache

        class Cache(dict):
            pass

        cache = Cache()

        class Chooser(object):
            implements(ICacheChooser)

            def __call__(self, key):
                assert key == 'plone.app.caching.operations.ramcache'
                return cache

        provideUtility(Chooser())

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['HTTP_ACCEPT_ENCODING'] = 'gzip; deflate'
        response.enableHTTPCompression(request)

        result = u"Body"
        response.setHeader('X-Foo', 'bar')

        IAnnotations(request)['plone.app.caching.operations.ramcache.key'] = 'foo'

        storeResponseInRAMCache(request, response, result)

        self.assertEqual(1, len(cache))
        cached = normalize_response_cache(cache['foo'])
        self.assertEqual((200, {'x-foo': 'bar'}, u'Body', 1), cached)

    def test_storeResponseInRAMCache_custom_keys(self):
        from plone.app.caching.operations.utils import storeResponseInRAMCache

        class Cache(dict):
            pass

        cache = Cache()

        class Chooser(object):
            implements(ICacheChooser)

            def __call__(self, key):
                assert key == 'cachekey'
                return cache

        provideUtility(Chooser())

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        result = u"Body"
        response.setHeader('X-Foo', 'bar')

        IAnnotations(request)['annkey'] = 'foo'

        storeResponseInRAMCache(request, response, result, globalKey='cachekey', annotationsKey='annkey')

        self.assertEqual(1, len(cache))
        cached = normalize_response_cache(cache['foo'])
        self.assertEqual((200, {'x-foo': 'bar'}, u'Body', 0), cached)

    # fetchFromRAMCache()

    def test_fetchFromRAMCache_no_cache(self):
        from plone.app.caching.operations.utils import fetchFromRAMCache

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['PATH_INFO'] = '/foo/bar'
        request.environ['QUERY_STRING'] = ''

        self.assertEqual(None, fetchFromRAMCache(request))

    def test_fetchFromRAMCache_minimal(self):
        from plone.app.caching.operations.utils import fetchFromRAMCache

        class Cache(dict):
            pass

        cache = Cache()

        class Chooser(object):
            implements(ICacheChooser)

            def __call__(self, key):
                assert key == 'plone.app.caching.operations.ramcache'
                return cache

        provideUtility(Chooser())

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['PATH_INFO'] = '/foo/bar'
        request.environ['QUERY_STRING'] = ''

        cache['http://example.com/foo/bar?'] = (200, {'x-foo': 'bar'}, u'Body')

        cached = normalize_response_cache(fetchFromRAMCache(request))
        self.assertEqual((200, {'x-foo': 'bar'}, u'Body'), cached)

    def test_fetchFromRAMCache_with_etag(self):
        from plone.app.caching.operations.utils import fetchFromRAMCache

        class Cache(dict):
            pass

        cache = Cache()

        class Chooser(object):
            implements(ICacheChooser)

            def __call__(self, key):
                assert key == 'plone.app.caching.operations.ramcache'
                return cache

        provideUtility(Chooser())

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['PATH_INFO'] = '/foo/bar'
        request.environ['QUERY_STRING'] = ''

        cache['||a|b||http://example.com/foo/bar?'] = (200, {'x-foo': 'bar'}, u'Body')

        cached = normalize_response_cache(fetchFromRAMCache(request, etag="|a|b"))
        self.assertEqual((200, {'x-foo': 'bar'}, u'Body'), cached)

    def test_fetchFromRAMCache_custom_key(self):
        from plone.app.caching.operations.utils import fetchFromRAMCache

        class Cache(dict):
            pass

        cache = Cache()

        class Chooser(object):
            implements(ICacheChooser)

            def __call__(self, key):
                assert key == 'cachekey'
                return cache

        provideUtility(Chooser())

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['PATH_INFO'] = '/foo/bar'
        request.environ['QUERY_STRING'] = ''

        cache['http://example.com/foo/bar?'] = (200, {'x-foo': 'bar'}, u'Body')

        cached = normalize_response_cache(fetchFromRAMCache(request, globalKey='cachekey'))
        self.assertEqual((200, {'x-foo': 'bar'}, u'Body'), cached)

    def test_fetchFromRAMCache_miss(self):
        from plone.app.caching.operations.utils import fetchFromRAMCache

        class Cache(dict):
            pass

        cache = Cache()

        class Chooser(object):
            implements(ICacheChooser)

            def __call__(self, key):
                assert key == 'plone.app.caching.operations.ramcache'
                return cache

        provideUtility(Chooser())

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['PATH_INFO'] = '/foo/bar'
        request.environ['QUERY_STRING'] = ''

        cache['http://example.com/foo/bar?'] = (200, {'x-foo': 'bar'}, u'Body')

        cached = normalize_response_cache(fetchFromRAMCache(request, etag='|foo'))
        self.assertEqual(None, cached)

    def test_fetchFromRAMCache_miss_custom_default(self):
        from plone.app.caching.operations.utils import fetchFromRAMCache

        class Cache(dict):
            pass

        cache = Cache()

        class Chooser(object):
            implements(ICacheChooser)

            def __call__(self, key):
                assert key == 'plone.app.caching.operations.ramcache'
                return cache

        provideUtility(Chooser())

        environ = {'SERVER_NAME': 'example.com', 'SERVER_PORT': '80'}
        response = HTTPResponse()
        request = HTTPRequest(StringIO(), environ, response)

        request.environ['PATH_INFO'] = '/foo/bar'
        request.environ['QUERY_STRING'] = ''

        cache['http://example.com/foo/bar?'] = (200, {'x-foo': 'bar'}, u'Body')

        marker = object()
        cached = normalize_response_cache(fetchFromRAMCache(request, etag='|foo', default=marker))
        self.assertTrue(cached is marker)



import zope.i18nmessageid

from zope.interface import Interface
from zope import schema

_ = zope.i18nmessageid.MessageFactory('plone.app.caching')

class ICacheProfiles(Interface):
    """Marker interface for extension profiles that contain cache settings.
    These will primarily include a ``registry.xml`` file to configure cache
    settings.

    To use the marker interface, you can do::

        <genericsetup:registerProfile
            name="my-cache-settings"
            title="My cache settings"
            directory="profiles/my-cache-settings"
            description="My cache settings"
            for="plone.app.caching.interfaces.ICacheProfiles"
            provides="Products.GenericSetup.interfaces.EXTENSION"
            />

    This will hide the profile from the Plone quickinstaller, and make it
    available for installation in the cache settings control panel.
    """

class IPloneCacheSettings(Interface):
    """Settings stored in the registry.

    Basic cache settings are represented by
    ``plone.caching.interfaces.ICacheSettings``. These are additional,
    Plone-specific settings.
    """

    templateRulesetMapping = schema.Dict(
            title=_(u"Page template/ruleset mapping"),
            description=_(u"Maps skin layer page template names to ruleset names"),
            key_type=schema.ASCIILine(title=_(u"Page template name")),
            value_type=schema.DottedName(title=_(u"Ruleset name")),
        )

    contentTypeRulesetMapping = schema.Dict(
            title=_(u"Content type/ruleset mapping"),
            description=_(u"Maps content type names to ruleset names"),
            key_type=schema.ASCIILine(title=_(u"Content type name")),
            value_type=schema.DottedName(title=_(u"Ruleset name")),
        )

    purgedContentTypes = schema.Tuple(
            title=_(u"Content types to purge"),
            description=_(u"List content types which should be purged when modified"),
            value_type=schema.ASCIILine(title=_(u"Content type name")),
            default=('File', 'Image', 'News Item', ),
        )

    cacheStopRequestVariables = schema.Tuple(
            title=_(u"Request variables that prevent caching"),
            description=_(u"Variables in the request that prevent caching if present"),
            value_type=schema.ASCIILine(title=_(u"Request variables")),
            default=('statusmessages', 'SearchableText',),
        )

class IETagValue(Interface):
    """ETag component builder

    Register a named multi-adapter from ``(published, request)`` to this
    interface to provide the values for ETag compnents. Various caching
    operations will look up such adapters to compose an ETag value. The
    adapter name is used in options configuring those components.
    """

    def __call__():
        """Return the ETag component, as a string.
        """

class IRAMCached(Interface):
    """Marker interface applied to the request if it should be RAM cached.

    The cache key will be stored in request annotations.
    """
